package be.ac.ulb.code.wit;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import javax.xml.xpath.XPathConstants;

import org.apache.commons.io.IOUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.glassfish.jersey.client.ClientConfig;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;

public class MapClient {
	public static final String GOOGLE_GEOCODE_REQ = "http://maps.googleapis.com/maps/api/geocode/xml?sensor=false&address=%s";
	public static final String GOOGLE_GEOCODE_STATUS = "GeocodeResponse/status";
	public static final String GOOGLE_GEOCODE_RESULT = "GeocodeResponse/result";
	public static final String GOOGLE_GEOCODE_RESULT_NAME = "address_component[type='route']/long_name";
	public static final String GOOGLE_GEOCODE_RESULT_LATITUDE = "geometry/location/lat/text()";
	public static final String GOOGLE_GEOCODE_RESULT_LONGITUDE = "geometry/location/lng/text()";
	public static final String GOOGLE_GEOCODE_RESULT_COUNTRY = "address_component[type='country']/long_name";
	public static final String GOOGLE_GEOCODE_RESULT_STATE = "address_component[type='administrative_area_level_1']/long_name";
	public static final String GOOGLE_GEOCODE_RESULT_CITY = "address_component[type='locality']/long_name";
	
	public static final String GOOGLE_IMAGERY_REQ = "http://maps.googleapis.com/maps/api/staticmap?center=%f,%f&zoom=%d&size=%s&sensor=false&format=%s&maptype=%s&language=fr";
	public static final String GOOGLE_IMAGESET_AERIAL = "satellite";
	public static final String GOOGLE_IMAGESET_ROAD = "roadmap";
	public static final String GOOGLE_IMAGESET_SIZE = "400x400";
	public static final String GOOGLE_FORMAT_PNG = "png";
	public static final Integer GOOGLE_ZOOM_LVL_MIN = 1;
	public static final Integer GOOGLE_ZOOM_LVL_MAX = 21;
	public static final Integer GOOGLE_ZOOM_LVL_DEFAULT = 14;

    /**
     * Parse responses from the Google Maps API, and return them
     * as a list of Location.
     */
	public List<Location> parseGoogleServiceResponse(InputStream response) {
        //Build a DOM representation of the document
		Document doc = XMLHelper.xml(response);
		ArrayList<Location> result = new ArrayList<Location>();

		// Check the error code
        String status = XMLHelper.xpath(doc, GOOGLE_GEOCODE_STATUS, XPathConstants.STRING);

        if (!status.equals("OK")) {
        	throw new RuntimeException("Google could not serve this request. ("+status+")");	
        }
		
        // Check each node in the result
        NodeList list = XMLHelper.xpath(doc, GOOGLE_GEOCODE_RESULT, XPathConstants.NODESET);
        for (int i = 0; i < list.getLength(); ++i) {                                                            
            result.add(parseLocation(list.item(i)));                                                            
        } 
        
		return result;
	}

    private Location parseLocation(Node item) {
    	String country = XMLHelper.xpath(item, GOOGLE_GEOCODE_RESULT_COUNTRY, XPathConstants.STRING);
    	
    	//TODO: Parse the remainder of the response and create a new Location object
    	//      using the data.
    	
    	throw new RuntimeException("Not implemented!");
	}

	/**
     * Find the actual locations corresponding to a given textual location
     */
	public List<Location> getLocations(String location) throws IOException {
		// Build the url with the proper location
		String url = String.format(GOOGLE_GEOCODE_REQ, URLEncoder.encode(location, "UTF-8"));
		
		Response response = null;
		InputStream data = null;
		
        //TODO: Send the query to Google Geocode, to find the location
		//		For this purpose, create a Client object, and
		//      retrieve the resource at the url we built.
		// 		Refer to 
		//			https://jersey.java.net/documentation/latest/client.html#d0e3827
		if (true) throw new RuntimeException("Not implemented!");
		
		// TODO: Extract the data from our response
		if (true) throw new RuntimeException("Not implemented!");
		
		//Note: Don't forget to close the data input stream.
		
        // Parse the data
		return parseGoogleServiceResponse(data);
	}
	
	private void saveResponse(String file, InputStream response) throws IOException {
		FileOutputStream fos = new FileOutputStream(file);
		IOUtils.copy(response, fos);
		fos.flush();
		fos.close();
	}

    /**
     * Download the map corresponding to a given location at
     * a specific zoom level to the specified file.
     */
	public void getMapImagery(Location loc, String outputfile, int zoomlevel) throws IOException {
        //TODO: Send the query to Static Maps, and fill the response object.
		InputStream response = null;
		
		if (true) throw new RuntimeException("Not implemented!");
		
		saveResponse(outputfile, response);
		//Note: Don't forget to close the image input stream.
	}

	public static void getMap(String location, String output, int zoomlevel) throws IOException {
		MapClient client = new MapClient();
		List<Location> locations;
		locations = client.getLocations(location);

		System.out.println("Found " + locations.size() + " results");
		for (Location loc: locations) {
			System.out.println("\t" + loc.toString());
		}

		if (output != null) {
			Location loc = locations.get(0);

			System.out.println("Saving map of " + loc.toString() + " to " + output);
			client.getMapImagery(loc, output, zoomlevel);
		}
	}

	public static void main(String[] args) throws IOException {
		String location = (args.length > 0) ? args[0] : null;
		String output = (args.length > 1) ? args[1]: null;
		int zoomlevel = 14;

		getMap(location, output, zoomlevel);
	}
}
